/* Copyright (c) 1997, MpegTV LLC */
#ifndef _com_api_h
#define _com_api_h
#ifndef lint
static const char rcsid_com_api_h[] = "$Header: /home/tristan/mpeg/dec/RCS/com_api.h,v 1.119 1999/09/16 00:22:04 tristan Exp tristan $";
#endif


/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "misc.h"
#include "pmp.h"
#include "alert_api.h"
#include "mode_api.h"
#include "audio_api.h"
#include "ts_api.h"
#ifdef VCD
#include "vcd_api.h"
#endif
#include "voh_api.h"
#include "state_api.h"

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/
/* MTVP SDK version */
#define MTVP_VERSION_MAJOR		1
#define MTVP_VERSION_MINOR		2
#define MTVP_VERSION_REVISION	1

/*
 * MTVP_API_VERSION was formerly COM_API_VERSION, but this was a very bad name
 * choice (could be confused with a COMMAND).
 */
#define MTVP_API_VERSION	18

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* values for MTVP_COMMAND_VIDEO_LUMA_FLOOR or Luma_floor */
#define MTVP_LUMA_FLOOR_MIN	(-128)
#define MTVP_LUMA_FLOOR_MAX	128
#define MTVP_LUMA_FLOOR_NORMAL	0

/* values for MTVP_COMMAND_VIDEO_LUMA_SHIFT or Luma_shift */
#define MTVP_LUMA_SHIFT_MIN	(-128)
#define MTVP_LUMA_SHIFT_MAX	128
#define MTVP_LUMA_SHIFT_NORMAL	0

/* values for MTVP_COMMAND_VIDEO_CHROMA_SATURATION or Chroma_correct */
#define MTVP_CHROMA_SATURATION_MIN	0
#define MTVP_CHROMA_SATURATION_MAX	256
#define MTVP_CHROMA_SATURATION_NORMAL	128

/* values for MTVP_COMMAND_VIDEO_GAMMA or Gamma_correct */
#define MTVP_GAMMA_MIN	0.5
#define MTVP_GAMMA_MAX	2.0
#define MTVP_GAMMA_NORMAL	1.0

/* values for MTVP_COMMAND_CHROMA_SHIFT_CR */
#define MTVP_CHROMA_SHIFT_CR_MIN	(-64)
#define MTVP_CHROMA_SHIFT_CR_MAX	64
#define MTVP_CHROMA_SHIFT_CR_NORMAL	0

/* values for MTVP_COMMAND_CHROMA_SHIFT_CB */
#define MTVP_CHROMA_SHIFT_CB_MIN	(-64)
#define MTVP_CHROMA_SHIFT_CB_MAX	64
#define MTVP_CHROMA_SHIFT_CB_NORMAL	0

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/*
 * Commands that can be sent to the Player.
 *
 * Most commands set parameters or ask the player to perform some action.
 * Other command (usually starting with COM_GET_...) ask the player for some data.
 *
 * Everything is quite well tested and has no reasons to change, except
 * the items marked [experimental].
 *
 * The type of command parameter is indicated first, in parenthesis.
 * (void) means that the command takes no parameter.
 * (custom) means that the command parameter is defined by the application.
 * (private) means that the message parameter type is private, i.e.
 * unknown from the application.
 *
 * reserved messages (like COM_SIH_SET) may be used privately
 * by the communication protocol and should not be used the application.
 *
 * deprecated messages (like COM_OPEN_FILE) will not be supported in future releases.
 * They are provided for compatibility only. Do not use them in new code!
 *
 * obsolete messages are not supported anymore.
 */
typedef enum {			/* (param) description */
	MTVP_COMMAND_MIN =0,			/* minimum value */
	MTVP_COMMAND_NONE =0,		/* reserved */
	MTVP_COMMAND_QUIT =2,		/* (void) quit */
	/* 3 obsolete */
	/* 4 obsolete */
	/* 5 obsolete */
	MTVP_COMMAND_CLOSE_STREAM =6,	/* (void) close stream previously opened by a MTVP_COMMAND_OPEN_... */
	MTVP_COMMAND_SAVE_STREAM =7,	/* (string) save copy of stream in file (if stream is seekable).  param is file name */
	MTVP_COMMAND_STATE =8,		/* (int) set state (param is a MTVP_STATE, must be MTVP_STATE_PLAYING or MTVP_STATE_PAUSED) */
	/* 9 obsolete */
	MTVP_COMMAND_END =10,		/* (void) seek to end of input if stream is seekable */
	MTVP_COMMAND_SEEK =11,		/* (int) seek to specified byte position (>= 0) if stream is seekable */
	MTVP_COMMAND_LOOP =12,		/* (int) set loop flag (param is a Bool) */
	MTVP_COMMAND_MSG_SELECT =14,	/* (int) set message mask (see msg_types.h) */
	MTVP_COMMAND_AUTO_EXIT =15,	/* (int) set auto_exit flag (param is a Bool) */
	MTVP_COMMAND_AUTO_PLAY =16,	/* (int) set auto_play flag (param is a Bool) */
	MTVP_COMMAND_VIDEO_DITHER_TYPE =17,	/* (int) set dither type (param is a MTVP_DITHER_TYPE) */
	MTVP_COMMAND_VIDEO_GAMMA =18,/* (double) set gamma correction (param > 0) */
	MTVP_COMMAND_SYNC_TYPE =19,	/* (int) set sync type (param is a MTVP_SYNC_TYPE) */
					/* if param is MTVP_SYNC_FRAMERATE, enable framerate control
					 * else disable framerate control */
	MTVP_COMMAND_SYNC_RATE_FACTOR =20,	/* (double) playback framerate factor (range is [MTVP_SYNC_RATE_FACTOR_MIN, MTVP_SYNC_RATE_FACTOR_MAX] or 1.0 =use nominal framerate) */
	MTVP_COMMAND_AUDIO_MUTE =21,	/* (int) set mute flag (param is a Bool) */
	MTVP_COMMAND_AUDIO_CHANNEL =22,	/* (int) set audio channel (param is an MTVP_AUDIO_CHANNEL) */
	MTVP_COMMAND_AUDIO_QUALITY =23,	/* (int) set audio quality (param is an MTVP_AUDIO_QUALITY) */
						/* if param is MTVP_AUDIO_QUALITY_LOW, disable audio channel control
					 	 * else enable audio channel control */
	MTVP_COMMAND_AUDIO_VOLUME =24,	/* (double) set volume (param range is [0.0, 1.0]) */
	MTVP_COMMAND_AUDIO_BALANCE =25,	/* (double) set balance (range is [0.0 =LEFT, 1.0 =RIGHT]) */
	MTVP_COMMAND_AUDIO_CONTROLS =26,	/* (int) select what volume and balance act on when a mixer is available (param is AUDIO_CONTROLS) */
	MTVP_COMMAND_WINDOW_CLOSE =27,	/* (void) close (iconify) player window [not implemented] */
	MTVP_COMMAND_WINDOW_OPEN =28,	/* (void) open player window (if iconified) [not implemented] */
	MTVP_COMMAND_SYS_FLUSH =31,		/* (void) flushes the systems and input buffers and tell player to look for the next random access entry point (generally a GOP header), like if a seek had just been performed [experimental] */
	MTVP_COMMAND_SYS_FLUSH_AFTER_NEW_STREAM =32,	/* (void) tell player to reset to default parsing (i.e. video stream) and to look for the first sequence header [experimental] */
	MTVP_COMMAND_SYS_READ_STRATEGY =33,	/* reserved */
	/* 34 obsolete */
	MTVP_COMMAND_SAVE_IMAGE_PPM =35,	/* (string) save copy of current frame in ppm format (param =filename) */
	MTVP_COMMAND_SAVE_IMAGE_TGA =36,	/* (string) save copy of current frame in tga format (param =filename) */
	MTVP_COMMAND_GET_POS =37,		/* (void) get current byte position of video decoder in stream */
	MTVP_COMMAND_GET_ACTUAL_FRAMERATE =38,	/* (void) get the actual framerate */
	MTVP_COMMAND_GET_NUMBER_OF_FRAMES_PLAYED =39,	/* (void) get the number of frames played since last seek */
	MTVP_COMMAND_AUDIO_MSG_SELECT =40,	/* (int) set audio process message mask (see msg_types.h) */
	MTVP_COMMAND_VIDEO_LUMA_FLOOR =41,	/* (double) set luma floor (i.e. contrast).  negative values increase contrast. */
	MTVP_COMMAND_VIDEO_LUMA_SHIFT =42,	/* (double) set luma shift (i.e. brightness) */
	MTVP_COMMAND_AUTO_SEEK =43,	/* (int) seek to specified byte position right after opening stream (if stream is seekable) [experimental] */
	MTVP_COMMAND_AUDIO_PORTS_SELECT =44,	/* (int) select audio output ports (param is a combination of AUDIO_PORT bit masks) */
	MTVP_COMMAND_LOOP_START_POS =47,		/* (int) set byte position of start of loop, or 0 [experimental] */
	MTVP_COMMAND_LOOP_END_POS =48,		/* (int) set byte position of end of loop, or 0 [experimental] */
	MTVP_COMMAND_AUDIO_SWAP_CHANNELS =49,	/* (int) swap right and left channels (param is a Bool) */
//	MTVP_COMMAND_VIDEO_FREEZE =51,		/* OBSOLETE - replaced by MTVP_COMMAND_VIDEO_SUSPEND and MTVP_COMMAND_VIDEO_RESUME */
	MTVP_COMMAND_SEND_GOP_INFOS =52,		/* (int) select whether MTVP_NOTIFY_GOP_INFOS messages will be received each time a gop is decoded when playing. 0 =no (default), 1 =yes */
	MTVP_COMMAND_CHALLENGE =53,		/* (private) reserved - answer to authentification request */
	MTVP_COMMAND_SIH_SET =55,		/* reserved */
	MTVP_COMMAND_SIH_LINK =56,		/* (string) link custom Stream Input Handler (param is the full pathname of the sih .so file). returns value =handler-id in MTVP_PMP_DATA_RETURN_SUCCESS */
	MTVP_COMMAND_VIDEO_SUSPEND =57,	/* (void) drain then suspend display [experimental] */
	MTVP_COMMAND_VIDEO_RESUME =58,	/* (void) resume display [experimental] */
	MTVP_COMMAND_DITHER_SPEC=59,	/* (int) param = MTVP_DITHER_SPEC_... - specify special dither for RGB conversion [experimental] */
	/* 60 obsolete */
	/* 61 obsolete */
	/* 62 obsolete */
	MTVP_COMMAND_STEPS =63,			/* (int) step forward N frames (param > 0) */
	MTVP_COMMAND_DEMO =64,			/* reserved */
	MTVP_COMMAND_VIDEO_ASPECT_RATIO =65,	/* (int) video aspect ratio (param is a MTVP_ASPECT_RATIO_TYPE) */
	MTVP_COMMAND_WINDOW_ID =66,		/* (int) specify window id (0 =create new window, -1 =use root window, other: use specified window) */
	MTVP_COMMAND_VIDEO_CHROMA_SATURATION =67,/* (int) set chroma saturation. range [MTVP_CHROMA_SATURATION_MIN, MTVP_CHROMA_SATURATION_MAX], default MTVP_CHROMA_SATURATION_NORMAL */
	MTVP_COMMAND_VIDEO_CHROMA_SHIFT_CR =68,/* (int) range [-64, 64], default 0; positive = more red, negative = more green */
	MTVP_COMMAND_VIDEO_CHROMA_SHIFT_CB =69,/* (int) range [-64, 64], default 0; positive = more blue, negative = more yellow */
	/* 70 obsolete */
	MTVP_COMMAND_OPEN_STREAM_URL =71,/* (string) open stream specified with URL */
	MTVP_COMMAND_SIH_INIT =72,		/* (custom) init SIH handler (param =handler-id followed by a URL-type string that will be passed to SIH_init) */
	MTVP_COMMAND_SIH_FREE =73,		/* (int) free SIH handler (param =handler-id) */
	MTVP_COMMAND_SIH_CNTL =74,		/* (custom) call cntl() routine of specified SIH handler (param =handler-id followed by a block of private data that will be passed to SIH_cntl) */
	/* 75 obsolete */
	MTVP_COMMAND_VOH_LINK =76,		/* (string) link custom Video Output Handler (param is the full pathname of the voh .so file). value set to handler-id in PMP_DATA_RETURN_SUCCESS [experimental] */
	MTVP_COMMAND_VOH_SELECT =77,		/* (int) select specified Video Output Handler (param is the handler-id) [experimental] */
	MTVP_COMMAND_VOH_CNTL =78,		/* (custom) call cntl() routine of specified VOH handler (param =handler-id followed by a block of private data that will be passed to VOH_cntl)  [experimental] */
	MTVP_COMMAND_VIDEO_REPAINT =79,	/* (void) if not playing, repaint the current frame [experimental] */
	MTVP_COMMAND_VIDEO_FOCUS =80,	/* (void) must be called on focus (windows only) [experimental] */
	MTVP_COMMAND_VOH_INIT =81,		/* (custom) instanciate specified VOH handler (param =handler-id (int32) followed by a string)  [experimental] */
	MTVP_COMMAND_VOH_FREE =82,		/* (custom) de-instanciate specified VOH handler (param =handler-id (int32))  [experimental] */
	MTVP_COMMAND_MAX				/* marker - follows maximum value defined in this version */
} MTVP_COMMAND;

/*
 * Types of notification messages that can be received from the Player.
 *
 * Those messages indicate a change of the state of the player, or a change
 * in the parameters of the stream being played, or deliver requested information.
 *
 * Everything is quite well tested and has no reasons to change.
 *
 * The type of message parameter is indicated first, in parenthesis.
 * (void) means that the message has no parameter.
 * (reserved) means that the message parameter type is private, i.e.
 * unknown from the application.
 *
 * reserved messages (like A2G_COM_API_VERSION) may be used privately
 * by the communication protocol and are not received to the application.
 */
typedef enum {
	MTVP_NOTIFY_MIN =1,				/* minimum value */
	MTVP_NOTIFY_API_VERSION =1,		/* (int) reserved - now caught by g2a_init() */
	MTVP_NOTIFY_RETURN_SUCCESS=2,	/* (MTVP_PMP_DATA_RETURN_SUCCESS) command with MTVP_PMP_FLAG_RETURN_STATUS flag set has returned success. A message of this type is immediately followed by a copy of the original message if the flag MTVP_PMP_FLAG_RETURN_ORIGINAL was set in the command */
	MTVP_NOTIFY_RETURN_FAILURE=3,	/* (MTVP_PMP_DATA_RETURN_FAILURE) command with MTVP_PMP_FLAG_RETURN_STATUS flag set has returned failure. A message of this type is immediately followed by a copy of the original message if the flag PMP_FLAG_RETURN_ORIGINAL was set in the command */
	MTVP_NOTIFY_READY=4,			/* (void) player is ready to take commands (i.e. inits are done) */
	MTVP_NOTIFY_QUIT=5,				/* 5: (void) quit */
	MTVP_NOTIFY_STATE=6,			/* (int) player's state changed (param is a MTVP_STATE) */
	MTVP_NOTIFY_POS=7,				/* (int) current byte position of video decoder in stream */
	MTVP_NOTIFY_ACTUAL_FRAMERATE=8,	/* (double) actual frame rate (0.0 if unknown) */
	MTVP_NOTIFY_NUMBER_OF_FRAMES_PLAYED=9,/* (int) number of frames played */
	MTVP_NOTIFY_AUDIO_ENABLED=10,	/* (void) audio enabled (i.e. audio will be played when playing video) */
	MTVP_NOTIFY_AUDIO_DISABLED=11,	/* (void) audio disabled (i.e. audio will not be played when playing video, either because no stream is opened, or the stream is a video stream or because sync mode or sync frame-rate does not permit audio play) */
	MTVP_NOTIFY_CANNOT_SHOW_VIDEO=12,	/* (int) Cannot show video. param is the error code (e.g. not enough memory to allocate window) */
	MTVP_NOTIFY_AUDIO_VOLUME=13,	/* (double) new volume (param range is [0.0, 1.0]) */
	MTVP_NOTIFY_AUDIO_BALANCE=14,	/* (double) new balance (range is [0.0 = LEFT, 1.0 = RIGHT]) */
	MTVP_NOTIFY_AUDIO_MUTE=15,		/* (void) audio has been temporarilly muted by player. This can happen if the driver does not support the selected sampling rate. Application can clear the mute flag with the MTVP_COMMAND_AUDIO_MUTE command to re-enable audio. */
	MTVP_NOTIFY_VIDEO_DITHER_TYPE=16,	/* (int) dither type used (param is a MTVP_DITHER_TYPE) */
	MTVP_NOTIFY_VIDEO_DITHER_FLAGS=17,	/* (int) flags indicating enabled dither type (param is equal to 0 or a 'or' combination of MTVP_VOH_MODE_GRAY, MTVP_VOH_MODE_ORDERED and MTVP_VOH_MODE_COLOR) */
	MTVP_NOTIFY_INPUT_FILE_SIZE=18,	/* (int) file size in bytes, or -1 if size unknown */
	MTVP_NOTIFY_DISPLAY_THREAD_ID=19,	/* [experimental] notify of the thread ID of the display thread */
	                                    /* win32: param is (int) containing the thread HANDLE. unix: param is (custom) that is a pthread_t) */
	MTVP_NOTIFY_WINDOW_CREATED=20,	/* (void) window has been successfully created. If the window couldn't be created, MTVP_NOTIFY_CANNOT_SHOW_VIDEO is sent */
	MTVP_NOTIFY_DISPLAY_READY=21,	/* (void) display ready (e.g. after seek, etc) */
	MTVP_NOTIFY_ERROR_IN_STREAM=22,	/* (int) found serious error in stream that triggered concealment (param = reserved) */
	MTVP_NOTIFY_VIDEO_INFOS=23,		/* (MTVP_PMP_DATA_VIDEO_INFOS) new video sequence infos */
	MTVP_NOTIFY_AUDIO_INFOS=24,		/* (MTVP_PMP_DATA_AUDIO_INFOS) new audio infos */
	MTVP_NOTIFY_FRAME_INFOS=25,		/* (MTVP_PMP_DATA_FRAME_INFOS) frame infos for new frame displayed. This message is received only when state != MTVP_STATE_PLAYING. */
	MTVP_NOTIFY_PACK_INFOS=26,		/* (MTVP_PMP_DATA_PACK_INFOS) new mux bitrate in bit/sec or 0 if VBR */
	MTVP_NOTIFY_VIDEO_COLORS_ALLOCATED=27,	/* (int) number of colors allocated in colormap */
	MTVP_NOTIFY_AUDIO_PORTS_AVAILABLE=28,	/* (int) available audio output ports (param is a combination of AUDIO_PORT bit masks) */
	MTVP_NOTIFY_AUDIO_PORTS_SELECTED=29,	/* (int) selected audio output ports (param is a combination of AUDIO_PORT bit masks) */
	/* 30 obsolete */
	MTVP_NOTIFY_LOOP_END=31,			/* (void) player reached end of loop and seeked back to beginning of loop */
	MTVP_NOTIFY_AUDIO_PID=32,			/* (int) pid of audio process, if using non-multithreaded version (unix only) */
	MTVP_NOTIFY_GOP_INFOS=33,			/* (MTVP_PMP_DATA_GOP_INFOS) gop infos */
	MTVP_NOTIFY_AUDIO_NO_CONTROLS_SELECT=34,	/* (void) audio mixer not available, MTVP_COMMAND_AUDIO_CONTROLS not supported */
	MTVP_NOTIFY_CHALLENGE=35,			/* (private) authentification request */
	MTVP_NOTIFY_VOH_FEEDBACK=36,		/* (custom) private feedback message from Video Output Handler */
	MTVP_NOTIFY_SIH_FEEDBACK=37,		/* (custom) private feedback message from Stream Input Handler */
	MTVP_NOTIFY_VCI_FEEDBACK=38,		/* (custom) private feedback message from Video Capture Interface */
	/* 39 obsolete */
	MTVP_NOTIFY_MPEG2_VIDEO=40,			/* (void) stream is MPEG2 (cannot play, has been closed) */
	MTVP_NOTIFY_BAD_STREAM=41,			/* (void) stream not MPEG1 (cannot play, has been closed) */
	MTVP_NOTIFY_CANNOT_PLAY_AUDIO=42,	/* (int) Cannot play audio (muting). param is the error code returned by the audio driver (e.g. device busy or sampling rate not supported) */
	MTVP_NOTIFY_RETURN_SYNCHRONOUS=43,	/* private notification, never received by application - (MTVP_PMP_DATA_RETURN_SYNCHRONOUS) command with MTVP_PMP_FLAG_SYNCHRONOUS flag set has returned */
	MTVP_NOTIFY_MAX=44				/* marker - follows maximum value defined in this version */
} MTVP_NOTIFY;

/* type of zoom */
typedef enum {
	MTVP_ZOOM_1X1 = 0,
	MTVP_ZOOM_2X2 = 1
} MTVP_ZOOM_TYPE;


/* type of aspect ratio for MTVP_COMMAND_VIDEO_ASPECT_RATIO */
typedef enum {
	MTVP_ASPECT_RATIO_SQUARE_PELS = 0,	/* use square pels */
	MTVP_ASPECT_RATIO_CODED = 1,		/* use coded pel_aspect_ratio */
	MTVP_ASPECT_RATIO_FREE = 2		/* always fill-up window */
} MTVP_ASPECT_RATIO_TYPE;

/* type of video sequence */
typedef enum {
	MTVP_VIDEO_MPEG1 = 1,
	MTVP_VIDEO_MPEG2 = 2		/* reserved */
} MTVP_VIDEO_TYPE;

/* video sequence infos */
typedef struct {
	MTVP_VIDEO_TYPE	type;
	u_int	width;				/* in pels */
	u_int	height;				/* in pels */
	u_int	bitrate;			/* in bytes/sec */
	u_int	vbv_buffer_size;	/* VBV buffer size in bytes */
	double	framerate;			/* in frames/sec */
	Bool	constrained_parameter_flag;
	double	pel_aspect_ratio;
} MTVP_PMP_DATA_VIDEO_INFOS;

/* gop infos */
typedef struct {
	Bool	time_code_drop;		/* time code, drop picture flag */
	u_int	time_code_hours;	/* time code, hours */
	u_int	time_code_minutes;	/* time code, minutes */
	u_int	time_code_seconds;	/* time code, seconds */
	u_int	time_code_pictures;	/* time code, pictures */
	Bool	closed_gop;			/* if True gop is closed */
	Bool	broken_link;		/* if True gop link is broken */
} MTVP_PMP_DATA_GOP_INFOS;

/* pack infos */
typedef struct {
	u_int	mux_rate;			/* new mux bitrate in bit/sec or 0 if VBR */
} MTVP_PMP_DATA_PACK_INFOS;

/* param for msg with type MTVP_NOTIFY_RETURN_SUCCESS */
typedef struct {
	MTVP_PMP_HEADER	original_header;	/* copy of header of original message */
#ifdef MTVP_THREAD
	void *original_data;	// malloc'ed copy of original data, must be freed upon reception if not NULL!!
#endif
	void 		*value;		/* some value (if msg type is MTVP_NOTIFY_RETURN_SUCCESS) */
} MTVP_PMP_DATA_RETURN_SUCCESS;

/* param for msg with type MTVP_NOTIFY_RETURN_FAILURE */
typedef struct {
	MTVP_PMP_HEADER	original_header;	/* copy of header of original message */
#ifdef MTVP_THREAD
	void *original_data;	// malloc'ed copy of original data, must be freed upon reception if not NULL!!
#endif
	int			error;				/* errno (if msg type is MTVP_NOTIFY_RETURN_FAILURE) */
} MTVP_PMP_DATA_RETURN_ERROR;

/* param for msg with type MTVP_NOTIFY_RETURN_SYNCHRONOUS */
typedef struct {
	MTVP_PMP_HEADER	original_header;	/* copy of header of original message */
	int			error;		/* 0 or a non-zero error code */
	void 		*value;		/* if error == 0, some return value */
} MTVP_PMP_DATA_RETURN_SYNCHRONOUS;

/* param for msg from player to control (union) */
typedef union {
	MTVP_PMP_DATA_GENERIC	data_generic;
	MTVP_PMP_DATA_RETURN_SUCCESS	data_return_success;
	MTVP_PMP_DATA_RETURN_ERROR	data_return_failure;
	MTVP_PMP_DATA_RETURN_SYNCHRONOUS	data_return_synchronous;
//	MTVP_PMP_DATA_ALERT	data_alert;
	MTVP_PMP_DATA_VIDEO_INFOS data_video_infos;
	MTVP_PMP_DATA_GOP_INFOS data_gop_infos;
	MTVP_PMP_DATA_FRAME_INFOS data_frame_infos;
	MTVP_PMP_DATA_AUDIO_INFOS data_audio_infos;
	MTVP_PMP_DATA_PACK_INFOS data_pack_infos;
	int data_sih_handler_id;
	int data_voh_handler_id;
	MTVP_VOH_FEEDBACK data_voh_feedback;
#ifdef USE_VOH_X
	MTVP_VOH_X_FEEDBACK data_voh_x_feedback;
#endif
} MTVP_PMP_DATA_A2G;


#endif /* _com_api_h */
