/*****************************************************************
|
|      Utility Library. Misc Functions
|
|      (c) 1996-1998 MpegTV, LLC
|      Author: Gilles Boccon-Gibod (gilles@mpegtv.com)
|
 ****************************************************************/

#ifndef __UTILS_H__
#define __UTILS_H__

/*----------------------------------------------------------------------
|       includes
+---------------------------------------------------------------------*/
#include "xa_config.h"

#ifndef XA_HAS_CTYPE
extern const unsigned char _ctype_table[];

#define _CTYPE_U	0x01	/* upper */
#define _CTYPE_L	0x02	/* lower */
#define _CTYPE_D	0x04	/* digit */
#define _CTYPE_C	0x08	/* cntrl */
#define _CTYPE_P	0x10	/* punct */
#define _CTYPE_S	0x20	/* white space (space/lf/tab) */
#define _CTYPE_X	0x40	/* hex digit */
#define _CTYPE_SP	0x80	/* hard space (0x20) */

#define isalnum(c) ((_ctype_table+1)[c]&(_CTYPE_U|_CTYPE_L|_CTYPE_D))
#define isalpha(c) ((_ctype_table+1)[c]&(_CTYPE_U|_CTYPE_L))
#define iscntrl(c) ((_ctype_table+1)[c]&(_CTYPE_C))
#define isdigit(c) ((_ctype_table+1)[c]&(_CTYPE_D))
#define isgraph(c) ((_ctype_table+1)[c]&(_CTYPE_P|_CTYPE_U|_CTYPE_L|_CTYPE_D))
#define islower(c) ((_ctype_table+1)[c]&(_CTYPE_L))
#define isprint(c) ((_ctype_table+1)[c]&(_CTYPE_P|_CTYPE_U|_CTYPE_L|_CTYPE_D|_CTYPE_SP))
#define ispunct(c) ((_ctype_table+1)[c]&(_CTYPE_P))
#define isspace(c) ((_ctype_table+1)[c]&(_CTYPE_S))
#define isupper(c) ((_ctype_table+1)[c]&(_CTYPE_U))
#define isxdigit(c) ((_ctype_table+1)[c]&(_CTYPE_D|_CTYPE_X))

#define isascii(c) (((unsigned) c)<=0x7f)
#define toascii(c) (((unsigned) c)&0x7f)

#endif /* not XA_HAS_CTYPE */

/*----------------------------------------------------------------------
|       prototypes
+---------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

extern void bytes_from_uint32(unsigned char *buffer, unsigned long value);
extern unsigned long bytes_to_uint32(const unsigned char *buffer);
extern void bytes_from_uint16(unsigned char *buffer, unsigned short value);
extern unsigned short bytes_to_uint16(const unsigned char *buffer);

/* C runtime functions */
#if defined(XA_HAS_MALLOC) && !defined(XA_HAS_CLOSEAPPS)
#define memory_alloc malloc
#else
extern void *memory_alloc(unsigned int);
#endif

#if defined(XA_HAS_CALLOC) && !defined(XA_HAS_CLOSEAPPS)
#define memory_calloc calloc
#else 
extern void *memory_calloc(unsigned int nb, unsigned int size);
#endif

#ifdef XA_HAS_FREE
#define memory_free free
#else
extern void memory_free(void *ptr);
#endif

#ifdef XA_HAS_MEMSET
#define memory_set memset
#else
extern void *memory_set(void *ptr, int c, unsigned int n);
#endif

#ifdef XA_HAS_MEMCPY
#define memory_copy memcpy
#else
extern void *memory_copy(void *dst, const void *src);
#endif

#ifdef XA_HAS_STRLEN
#define string_length strlen
#else
extern unsigned int string_length(const char *s);
#endif

#ifdef XA_HAS_STRDUP
#define string_duplicate strdup
#else 
extern char *string_duplicate(const char *);
#endif

#ifdef XA_HAS_STRCPY
#define string_copy strcpy
#else
extern char *string_copy(char *dst, const char *src);
#endif

#ifdef XA_HAS_STRNCPY
#define string_copy_n strncpy
#else
extern char *string_copy_n(char *dst, const char *src, unsigned int n);
#endif

#ifdef XA_HAS_STRCMP
#define strings_equal(s1,s2) (!strcmp(s1,s2))
#else
extern int strings_equal(const char *s1, const char *s2);
#endif

#ifdef XA_HAS_STRNCMP
#define strings_compare_n strncmp
#else
extern int strings_compare_n(const char *s1, const char *s2, unsigned int n);
#endif

#ifdef XA_HAS_STRCASECMP
#define strings_compare_no_case strcasecmp
#else
extern int strings_compare_no_case(const char *s1, const char *s2);
#endif

#ifdef XA_HAS_STRNCASECMP
#define strings_compare_n_no_case strncasecmp
#else
extern int strings_compare_n_no_case(const char *s1, const char *s2, unsigned int);
#endif

#ifdef XA_HAS_STRCHR
#define string_find_char strchr
#else
extern char *string_find_char(const char *s, int c);
#endif

#ifdef XA_HAS_STRTOUL
#define string_to_unsigned_long strtoul
#else
extern unsigned long string_to_unsigned_long(const char *string, 
                                             char **endp, 
                                             unsigned int base);
#endif

#ifdef XA_HAS_SPRINTF
#define string_format sprintf
#else
extern int string_format(char *buffer, const char *format, ...);
#endif

#ifdef XA_HAS_SSCANF
#define string_scan sscanf
#else
extern int string_scan(const char *s, const char *format, ...);
#endif

#ifndef XA_HAS_SETJMP_H
typedef long jmp_buf;
#endif /* XA_HAS_SETJMP_H_ */

#ifdef __cplusplus
}
#endif
 
#endif /* __UTILS_H__ */


