/*****************************************************************
|
|      MTVP Player Wrapper Class for win32   
|
|
|      (c) 2000 MpegTV, LLC
|      Author: Tristan Savatier (tristan@mpegtv.com)
|              Gilles Boccon-Gibod (gilles@mpegtv.com)
|
|      This class implements a thin C++ wrapper around
|      the MTVP ASYNC API.
|      A typical player impementation would subclass this base 
|      class, and overload some of the stubbed notification 
|      funtions.
|      To send commands to the player, just use the functions
|      inherited from MTVP_Player.
|      To receive notification, overload the notification
|      functions in the subclass to implement your own 
|      notication handlers.
|
 ****************************************************************/

#ifndef _mtvp_player_win32_hpp
#define _mtvp_player_win32_hpp

/*----------------------------------------------------------------------
|       includes
+---------------------------------------------------------------------*/
#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include "mtvp/mtvp_player.hpp"

/*----------------------------------------------------------------------
|       constants
+---------------------------------------------------------------------*/
extern "C" {
extern LRESULT
MTVP_Player_win32_message_handler(HWND window, 
                     UINT win32_message, 
                     WPARAM win32_wparam, 
                     LPARAM win32_lparam);
}

/*----------------------------------------------------------------------
|       MTVP_Player_win32 Class
+---------------------------------------------------------------------*/
class MTVP_Player_win32 : public MTVP_Player
{
public:
    MTVP_Player_win32(const char **mtvp_argv, HINSTANCE instance)
	{
		/* register a window class */
		LPCTSTR   class_name = TEXT("MpegTV Hidden Window Class");
		WNDCLASS               wclass;
		wclass.style         = 0;
		wclass.lpfnWndProc   = (long (__stdcall *)(struct HWND__ *,unsigned int,unsigned int,long))MTVP_Player_win32_message_handler;
		wclass.cbClsExtra    = 0;
		wclass.cbWndExtra    = 0;
		wclass.hInstance     = instance;
		wclass.hIcon         = 0;
		wclass.hCursor       = 0;
		wclass.hbrBackground = 0;
		wclass.lpszMenuName  = 0;
		wclass.lpszClassName = class_name;

		/* register the class and ignore any error because we might */
		/* be registering the class more than once                  */
		RegisterClass(&wclass);

		/* create a hidden window to receive notification messages */
		m_HiddenWindow = CreateWindow(
			class_name,                 // pointer to registered class name 
			TEXT("MpegTV Hidden Window"),// pointer to window name 
			0,                          // window style 
			0,                          // horizontal position of window 
			0,                          // vertical position of window 
			0,                          // window width 
			0,                          // window height 
			0,                       // handle to parent or owner window 
			0,                       // handle to menu or child-window identifier 
			instance,                   // handle to application instance 
			0);

		if (m_HiddenWindow) {
			/* set the window's user data to this object */
			SetWindowLong(m_HiddenWindow, GWL_USERDATA, (LONG)this);
		} else {
    		// we should probably throw an exception here
		}
//		voh_handler_id = MTVP_VOH_HANDLER_ID_WIN32;

		/* instanciate the XA_Player object */

		this->MtvpRun(mtvp_argv, "", (void *)m_HiddenWindow);
	}
   ~MTVP_Player_win32()
	{
		/* delete the hidden window */
		DestroyWindow(m_HiddenWindow);
	}

private:
    HWND m_HiddenWindow;
};

#endif /* _mtvp_player_win32_hpp */
