
#if !defined(_pockettv_h)
#define _pockettv_h

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "resource.h"
#include "commctrl.h"

#include "mtvp/mtvp.h"

#include "mtvp_player_win32.hpp"

/*--------------------------------------------------------------------------+
| Configuration flags
+--------------------------------------------------------------------------*/
// if you need to use those macros, it is safer to define them in the project file.
// #define NO_REGISTRATION	// remove registration code
// #define USE_FULLSCREEN	// add code that deals with full screen (only works on ipaq)

/*--------------------------------------------------------------------------+
| PocketTV Class
+--------------------------------------------------------------------------*/
#define POCKETTV_VERSION_MAJOR		0
#define POCKETTV_VERSION_MINOR		4
#define POCKETTV_VERSION_REVISION	6

typedef enum {
	UNDIM_TYPE_GUESS = -1,
	UNDIM_TYPE_NONE = 0,
	UNDIM_TYPE_HP_540 = 1,
	UNDIM_TYPE_CASIO_115 = 2
} UNDIM_TYPE;

typedef struct {
	UNDIM_TYPE	type;
	unsigned int		timer_interval;
} UNDIM;

class PocketTV : public MTVP_Player_win32
{
public:

typedef struct {
	bool casio_display_16bit_write;	// set if vram must be accessed with 16-bit writes
	bool casio_stretch_bug;			// set when StretchBlt is broken (does not do clipping properly)
	bool no_microdrive;				// set for those devices that cannot take a microdrive
	bool ipaq_audio_swapped_channels;
} DEVICE_SPEC;

typedef enum {
	LAST_OPEN_FROM_NONE,
	LAST_OPEN_FROM_PARAMETER,
	LAST_OPEN_FROM_OPEN_FILE,
	LAST_OPEN_FROM_OPEN_URL,
} LAST_OPEN_MODE;

typedef enum {
//	DISPLAY_SETTING_BRIGHTNESS,
//	DISPLAY_SETTING_CONTRAST,
	DISPLAY_SETTING_GAMMA,
	DISPLAY_SETTING_CHROMA_SATURATION,
	DISPLAY_SETTING_GREEN_RED,
	DISPLAY_SETTING_YELLOW_BLUE,
	DISPLAY_SETTING_COUNT,
} DISPLAY_SETTING;

typedef struct {
	const TCHAR *caption;	// caption
	const TCHAR *registry;	// name of registry value
	int		min;
	int		max;
	int		normal_value;	// range [min..max] - used when reset
	int		default_value;	// range [min..max] - used when reset
	int		value;			// range [min..max]
} DISPLAY_SETTING_INFO;

private:
	HWND				m_MenuCommandBar;		// The menu command bar handle
#ifndef POCKET_PC
	HWND				m_CommandBands;		// The command band handle
#endif
	HWND				m_ControlsCommandBar;	// The command bar handle that hold the slider
	HWND				m_StatusWindow;		// The status window handle
	bool				m_ShowStatusBar;	// Show/hide status bar
	HMENU				m_Menu;				// The menu bar handle
	TCHAR			   *m_NavBarTitle;		// name of opened stream as shown in navigation bar (when playing)
	bool				m_DisplayReady;		// turns True once first image is displayed after a seek
	int					m_PendingSeek;		// -1 or pending seek position
	HWND				m_SeekSlider;		// The track slider handle
	HWND				m_VolumeSlider;		// The volume slider handle
	HWND				m_DisplaySlider;	// The display slider handle
	int					m_DisplaySetting;	// The current display setting being adjusted
	bool				m_PendingDisplaySlider;	// don't set new setting while this is true
	int					m_PendingDisplaySliderPos;	// pending slider position for display setting
	DISPLAY_SETTING_INFO m_DisplaySettingInfo[DISPLAY_SETTING_COUNT];
	MTVP_STATE			m_StateBeforeSeeking;
	MTVP_STATE			m_State;			// current state on player
	int					m_WasPlayingBeforeBecommingInactive; // -1 means unknown (i.e. not yet set)
//	MTVP_STATE			m_StateBeforeCancelMode; // -1 means unknown (i.e. not yet set)
	bool				m_CancelModePending;
	bool				m_SettingVolume;	// used by volume slider procedure
	bool				m_FastForward;		// True if playing fast forward
    WNDPROC				m_VolumeSliderParentOriginalProc;
    WNDPROC				m_DisplaySliderParentOriginalProc;
	WNDPROC				m_SeekSliderParentOriginalProc;
	RECT				m_TargetRect;			// Rectangle for video
	double				m_FilesizeOverTrackSliderMax;
	double				m_TrackSliderMaxOverFilesize; // ratio of TrackSliderMax divided by Filesize
	bool				m_Seeking;			// True means we are currently seeking
	LONGLONG			m_LastPositionTimerTime;	// used by the position timer
	LONGLONG			m_LastVolumeSettingTime;
	bool				m_PlayerReady;		// turns True once the player is fully initialized
	bool				m_Muted;			// audio is muted
	int					m_VOHHandlerId;
	int					m_SIHHandlerId;
	bool				m_IgnoreCancelMode;
//	bool				m_Done;				// set when user clicks the OK button in the Nav Bar
	bool				m_AutoPlay;
	bool				m_FastVideo;		// Will use fast video if possible
//	bool				m_FastVideoDisabled;	// Fast video is not available
	VOH_WIN32_STRETCH	m_Stretch;
	UNDIM_TYPE			m_UndimType;
//	unsigned int		m_VolumeRange;		// divisor factor for the volume range (volume is too loud on the iPaq)
	double				m_Volume;			// current Volume (<0 means unknown)
	bool				m_VolumeDialogVisible;
	bool				m_DisplayDialogVisible;
	int					m_DisplayDialogTop;
	HANDLE				m_DisplayThread;			
	bool				m_StreamOpened;		// if false, no stream is currently opened
	bool				m_Registered;		// registration code + key correct
	bool				m_InOpenDialog;
#ifndef POCKET_PC
	TCHAR			   *m_StreamName;		// URL of opened stream
	RECT				m_StreamNameRect;	// Rectangle for file name
#endif
	bool				m_GapiExchangeXY;
	bool				m_DisableResumeVideo;
#ifdef USE_FULLSCREEN
	HWND				m_FullscreenWindow;
	bool				m_PlayFullscreen;
#endif
	double				m_RateFactor;				// playback rate factor for >>

	MTVP_PMP_DATA_PACK_INFOS m_PackInfos;
	MTVP_PMP_DATA_VIDEO_INFOS m_VideoInfos;
	unsigned int		m_Filesize;
	unsigned int		m_Position;
	double				m_ActualFramerate;
	DEVICE_SPEC			m_DeviceSpec;
//	bool				m_StartedWithParameter;		// current stream was opened via an .m1u (mpeg url) file for streaming
	TCHAR				*m_TrackbarClass;
//	HWND				m_FirstWindowThatLostFocus;
	HACCEL				m_AccelTable;
	bool				m_IsActive;			// true means main window is active
	bool				m_Microdrive;		// true means optimize for microdrive
	int					m_PendingOpens;		// number of pending "open stream" commands

public:
	HWND 				m_Window;
	HWND 				m_WasForegroundWindow;
	HINSTANCE			m_Instance;			// The current instance
	HWND				m_VolumeDialog;		// The volume dialog handle
	HWND				m_DisplayDialog;	// The display dialog handle
	const TCHAR		   *m_URLName;			// name of last opened URL
	DWORD				m_Code;				// registration code
#ifdef USE_FULLSCREEN
	bool				m_Fullscreen;
#endif
	bool				m_Dither;
	int					m_DitherType;	// MTVP_DITHER_SPEC_12BIT or MTVP_DITHER_SPEC_15BIT
	bool				m_MustRegisterNow;	// require immediate registration (disable the [Register Later] button)
	HWND				m_PreviouslyActiveWindow;
	WNDPROC				m_OriginalTrackbarWindowProc;
	LAST_OPEN_MODE		m_LastOpenMode;


public:
    PocketTV(const char **mtvp_argv, HINSTANCE instance, HWND hWnd) :
		MTVP_Player_win32(mtvp_argv, instance),
		m_Instance(instance),
		m_Window(hWnd),
		m_PendingSeek(-1),
		m_WasPlayingBeforeBecommingInactive(-1),
//		m_StateBeforeCancelMode(MTVP_STATE_UNKNOWN),
		m_CancelModePending(false),

		m_MenuCommandBar(0),
#ifndef POCKET_PC
		m_CommandBands(0),
		m_StreamName(NULL),
#endif
		m_ControlsCommandBar(0),
		m_StatusWindow(0),
		m_ShowStatusBar(false),
		m_Menu(0),
//		m_StreamNameRect(0),
		m_NavBarTitle(NULL),
		m_DisplayReady(false),
		m_SeekSlider(0),
		m_VolumeSlider(0),
		m_DisplaySlider(0),
		m_DisplaySetting(0),
		m_PendingDisplaySlider(false),
		m_PendingDisplaySliderPos(-1),
		m_StateBeforeSeeking(MTVP_STATE_PAUSED),
		m_State(MTVP_STATE_PAUSED),
		m_SettingVolume(false),
		m_FastForward(false),
		m_VolumeSliderParentOriginalProc(NULL),
		m_DisplaySliderParentOriginalProc(NULL),
		m_SeekSliderParentOriginalProc(NULL),
//		m_TargetRect(0),
		m_FilesizeOverTrackSliderMax(0.0),
		m_TrackSliderMaxOverFilesize(0.0),
		m_Seeking(false),
		m_LastPositionTimerTime(0),
		m_LastVolumeSettingTime(0),
		m_PlayerReady(false),
		m_Muted(false),
		m_VOHHandlerId(MTVP_VOH_HANDLER_ID_WIN32),
		m_SIHHandlerId(-1),
		m_IgnoreCancelMode(false),
//		m_Done(false),
		m_FastVideo(false),
//		m_FastVideoDisabled(false),
		m_AutoPlay(false),
		m_Stretch(VOH_WIN32_STRETCH_AUTO),
		m_UndimType(UNDIM_TYPE_GUESS),
//		m_VolumeRange(1),
		m_Volume(-1.0),
		m_VolumeDialogVisible(false),
		m_DisplayDialogVisible(false),
		m_DisplayDialogTop(0),
		m_DisplayThread(0),
		m_StreamOpened(false),
		m_MustRegisterNow(false),
		m_Registered(false),
		m_InOpenDialog(false),
		m_DisableResumeVideo(false),
#ifdef USE_FULLSCREEN
		m_FullscreenWindow(NULL),
		m_PlayFullscreen(false),
		m_Fullscreen(false),
#endif
		m_DitherType(MTVP_COLOR16_DITHER_NONE),
		m_Dither(false),
		m_RateFactor(8),


//		m_PackInfos(0),
//		m_VideoInfos(0),
		m_Filesize(0),
		m_Position(0),
		m_ActualFramerate(0.0),
		m_PreviouslyActiveWindow(NULL),
		m_WasForegroundWindow(NULL),
		m_VolumeDialog(0),
		m_DisplayDialog(0),
		m_URLName(NULL),
		m_Code(0),
		m_GapiExchangeXY(false),
		m_OriginalTrackbarWindowProc(NULL),
		m_TrackbarClass((TCHAR *)TRACKBAR_CLASS),
//		m_StartedWithParameter(false),
		m_LastOpenMode(LAST_OPEN_FROM_OPEN_FILE),
		m_AccelTable(NULL),
		m_IsActive(true),
		m_PendingOpens(0),
		m_Microdrive(false)
//		m_FirstWindowThatLostFocus(NULL)
	{
#ifndef POCKET_PC
		memset((void *)&m_StreamNameRect, 0, sizeof(m_StreamNameRect));
#endif
		memset((void *)&m_TargetRect, 0, sizeof(m_TargetRect));
		memset((void *)&m_PackInfos, 0, sizeof(m_PackInfos));
		memset((void *)&m_VideoInfos, 0, sizeof(m_VideoInfos));
		memset((void *)&m_DeviceSpec, 0, sizeof(m_DeviceSpec));
	};
   ~PocketTV();

   bool HandleKeyMessage(UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT HandleWindowsMessage(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT SeekSliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT VolumeSliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT DisplaySliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   BOOL VolumeDialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   BOOL DisplayDialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	void StartPlayTimers();
	void StopPlayTimers();
	int InitControls();
	void DestroyControls();
	void SetStreamControlsState(bool enabled);
	void ActivateStreamControls();
	void DeactivateStreamControls();
	void OpenURL(const TCHAR *url);
	void OpenURLEx(const TCHAR *url, HWND window, LAST_OPEN_MODE lastOpenMode);
	void OpenFile();
	void CloseStream();
	bool IsStretched(VOH_WIN32_STRETCH stretch);
	void SetVideoStretch0(VOH_WIN32_STRETCH stretch);
	void SetVideoStretchNoWarning(VOH_WIN32_STRETCH stretch);
	void SetVideoStretch(VOH_WIN32_STRETCH stretch);
	void SetAudioChannel(MTVP_AUDIO_CHANNEL channel);
//	void SetAudioSwapChannels(bool yes);
	void SetAudioQuality(MTVP_AUDIO_QUALITY quality);
	void Loop(bool yes);
	void AutoPlay(bool yes);
	void ShowStatusBar(bool yes);
	int GapiInit();
	void GapiSetVideoTargetRect();
	bool Gapi0(bool yes, bool sync);
	void FastVideo(bool yes);
//	int GapiSuspend();
//	int GapiResume();
	void SetState(MTVP_STATE state);
	void ReflectState(MTVP_STATE state);
	bool WasPlaying();
	int GetElapsedSeconds(unsigned int pos);
	void GetPositionParameters();
	void ShowTimeTotal();
	void ShowTimeElapsed();
	void ShowTimePosition(unsigned int pos);
	void ShowActualFramerate();
	void ShowBitrate();
	void MySeek(unsigned int pos);
	void KeepAlive();
	void CalculateVideoTargetRect();
	void SetVideoTargetRect();
	void UpdateVideoTargetRect();
	void UpdateSlidersSize();
	void UpdateSliderSize(HWND slider, HWND command_bar);
	HWND CreateSlider(HWND window, DWORD style, int x, int y, int width, int height, WNDPROC *window_original_proc, WNDPROC slider_proc);
	HWND CreateSliderInCommandBar(HWND command_bar, unsigned int last_button_index, WNDPROC *command_bar_original_proc, WNDPROC slider_proc);
	void ShowVolumeDialog();
	void HideVolumeDialog();
	void ShowDisplayDialog();
	void HideDisplayDialog();
	void MySetVolume(int volume);
	void ErrorBoxEx(HWND window, int ids_window_caption_error, const TCHAR *format, const va_list *args);
	void ErrorBox(HWND window, int ids_window_caption_error, const TCHAR *format, ...);
	void SetMute(bool yes);
#ifdef POCKET_PC
	void KeepBright();
	HWND CreateMenuBar(HWND hWnd, int nToolBarId, int dwFlags);
#endif
#ifndef NO_REGISTRATION
	void InitRegistration();
#endif
	void SetUndimType(UNDIM_TYPE type);
//	void SetVolumeRange(unsigned int range);
	void DialogAboutEasterEgg(HWND hWnd);
#ifdef USE_FULLSCREEN
	void Fullscreen(bool yes);
	void FullscreenKillFocus();
	void FullscreenSetFocus();
	void PlayFullscreen(bool yes);
#endif
//	int SuspendVideo0(unsigned int n);
//	int ResumeVideo0(unsigned int n);
	void SuspendVideo();
	void ResumeVideo();
	void Dither(bool yes);
	void SetDither(bool yes);
	void DitherType(int dither);
	void EnableDither(VOH_WIN32_STRETCH stretch);
	void Microdrive(bool yes);
	void SetMicrodrive(bool yes);
	void AllocateMicrodriveBuffer(bool yes);
	void EnableMicrodriveOptimization(bool yes);
	void RateFactor(int rate_factor);
	void SetRateFactor(int rate_factor);
//	bool IsCasioEM500();
	void InitDeviceSpec();
	void FixSliders();
	int GetURLFromM1UFile(const TCHAR *filename, char **purl);
	void HandleMessage(MSG *msg);
//	void HandlePendingMessages();
	void MakeRichInkControl(HWND hDlg, DWORD id, TCHAR *new_text);
	void InitDisplaySettingInfo();
	int CreateDisplayDialog();
	void ShowDisplaySetting();
	void SetDisplaySetting(int pos);
	void SetDisplaySetting0(int i, int pos);
	void SetDefaultDisplaySettings();
	void SetNormalDisplaySettings();
	void GapiRotate(bool yes);

	void MtvpOnNotifyState(MTVP_STATE state);
	void MtvpOnNotifyPackInfos(const MTVP_PMP_DATA_PACK_INFOS *ptr);
	void MtvpOnNotifyVideoInfos(const MTVP_PMP_DATA_VIDEO_INFOS *ptr);
	void MtvpOnNotifyFileSize(unsigned int size);
	void MtvpOnNotifyActualFramerate(double x);
	void MtvpOnNotifyVolume(double x);
	void MtvpOnNotifyPosition(int pos);
	void MtvpOnNotifyDisplayReady();
	void MtvpOnNotifyCannotShowVideo(int status);
	void MtvpOnNotifyCannotPlayAudio(int status);
	void MtvpOnNotifyReady();
	void MtvpOnNotifyDisplayThreadId(HANDLE thread);
	void MtvpOnNotifySuccess(MTVP_COMMAND command, void *value, const void *ptr, u_int size);
	void MtvpOnNotifyFailure(MTVP_COMMAND command, int status, const void *ptr, u_int size);
	void MtvpOnNotifyWindowCreated();
};

/*--------------------------------------------------------------------------+
| Globals
+--------------------------------------------------------------------------*/
extern PocketTV *Player;	// that is the only global

/*----------------------------------------------------------------------
|       prototypes
+---------------------------------------------------------------------*/

/*----------------------------------------------------------------------
|       macros
+---------------------------------------------------------------------*/

#define POCKETTV_HANDLE_WINDOW_MESSAGE(object_class,                     \
                                 handler,                          \
                                 window,                           \
						         message,                          \
						         wparam,                           \
						         lparam)                           \
{                                                                  \
	PocketTV *target = Player;                                     \
    if (target) {												   \
        return target->##handler##(window,                         \
		                           message,                        \
		 	    				   wparam,                         \
								   lparam);                        \
	} else {                                                       \
		return DefWindowProc(window, message, wparam, lparam);     \
	}                                                              \
}

extern "C" {
extern LRESULT CALLBACK DialogAbout(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
}

#endif // !defined(_pockettv_h)
