/*----------------------------------------------------------------------
|       includes
+---------------------------------------------------------------------*/
#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"

#include <commdlg.h>
#include <tchar.h>
#include <winnls.h>

#include "utils.h"

#if defined(UNDER_CE) && (UNDER_CE >= 212)

// pockettv_malloc and pockettv_calloc are smarter replacements for malloc and calloc.
//
// They try to free some memory by asking other applications to terminate gracefully 
// with SHCloseApps if there is not enough memory available.

/*----------------------------------------------------------------------
| pockettv_malloc
+---------------------------------------------------------------------*/
void *pockettv_malloc(unsigned int size)
{
    void *memory;

    if (memory = malloc(size)) {
        return memory;
    }
	SHCloseApps(size);

    return malloc(size);
}

/*----------------------------------------------------------------------
| pockettv_calloc
+---------------------------------------------------------------------*/
void *pockettv_calloc(unsigned int num, unsigned int size)
{
    void *memory;

    if (memory = calloc(num, size)) {
        return memory;
    }
	SHCloseApps(num*size);

    return calloc(num, size);
}

#endif // defined(UNDER_CE) && (UNDER_CE >= 212)


#ifdef SPI_SETSIPINFO

/*----------------------------------------------------------------------
| HideSip
+---------------------------------------------------------------------*/
BOOL
HideSip( void )
{   
    BOOL fRes = FALSE;
    SIPINFO si;
    memset( &si, 0, sizeof( si ) );
    si.cbSize = sizeof( si );
    if( SHSipInfo( SPI_GETSIPINFO, 0, &si, 0 ) )
    {
        si.fdwFlags &= ~SIPF_ON;
        fRes = SHSipInfo( SPI_SETSIPINFO, 0, &si, 0 );
    }    
    return fRes;
}

/*----------------------------------------------------------------------
| ShowSip
+---------------------------------------------------------------------*/
BOOL
ShowSip( void )
{   
    BOOL fRes = FALSE;
    SIPINFO si;
    memset( &si, 0, sizeof( si ) );
    si.cbSize = sizeof( si );
    if( SHSipInfo( SPI_GETSIPINFO, 0, &si, 0 ) )
    {
        si.fdwFlags |= SIPF_ON;
        fRes = SHSipInfo( SPI_SETSIPINFO, 0, &si, 0 );
    }    
    return fRes;
}

#endif

#if defined(UNDER_CE) && (UNDER_CE >= 212)
/*--------------------------------------------------------------------------+
| SetNavBarText
+--------------------------------------------------------------------------*/
int
SetNavBarText(HWND hWnd, const TCHAR *str)
{
//	return SHSetNavBarText(hWnd, str);
//#if 0
	int		status;

	if (str == NULL) {
		TCHAR buf[30];

		GetWindowText(hWnd, buf, sizeof(buf)/sizeof(buf[0]));
		status = SHSetNavBarText(hWnd, buf);
	} else {
		status = SHSetNavBarText(hWnd, str);
	}
	return status;
//#endif // 0
}
#endif


/*--------------------------------------------------------------------------+
| MyLoadString
+--------------------------------------------------------------------------*/
const TCHAR *
MyLoadString(HINSTANCE instance, int id)
{
	unsigned int n;
	TCHAR *q;
	const TCHAR *p = (const TCHAR *)LoadString(instance, id, 0, 0);
	if (p == NULL) {
		return NULL;
	}
	n = *((WORD *)p - 1);
	q = (TCHAR *)malloc((n + 1) * sizeof(TCHAR));
	if (q == NULL) {
		return NULL;
	}
	LoadString(instance, id, q, n + 1);
//	q[n] = 0;
	return q;
}

/*--------------------------------------------------------------------------+
| MyFreeString
+--------------------------------------------------------------------------*/
void
MyFreeString(const TCHAR *str)
{
	if (str != NULL) {
		free((void *)str);
	}
}
