// SimplePlayer
//

#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"
#include "simpleplayer.hpp"

#include <windowsx.h>
#include <commctrl.h>
#include <commdlg.h>
#include <winnls.h>
#include <shlobj.h>
#ifdef HAVE_AYGSHELL
#include <aygshell.h>
#endif

// WM_NCACTIVATE seems to be missing from the POCKET_PC header files
#ifndef WM_NCACTIVATE
#define WM_NCACTIVATE 0x86
#endif

////////////////////////// SHGetMenu Macro's
#ifdef POCKET_PC //Rapier devices

//Menu Bar Height
#define MENU_HEIGHT 26

#define SHGetMenu(hWndMB)  (HMENU)SendMessage((hWndMB), SHCMBM_GETMENU, (WPARAM)0, (LPARAM)0);
#define SHGetSubMenu(hWndMB,ID_MENU) (HMENU)SendMessage((hWndMB), SHCMBM_GETSUBMENU, (WPARAM)0, (LPARAM)ID_MENU);
#define SHSetSubMenu(hWndMB,ID_MENU) (HMENU)SendMessage((hWndMB), SHCMBM_SETSUBMENU, (WPARAM)0, (LPARAM)ID_MENU);

//Alternate definition
#define SHMenuBar_GetMenu(hWndMB,ID_MENU) (HMENU)SendMessage((hWndMB), SHCMBM_GETSUBMENU, (WPARAM)0, (LPARAM)ID_MENU);


#else //non-Rapier devices

#define MENU_HEIGHT 0

#define SHGetMenu(hWndCB) (HMENU)CommandBar_GetMenu(hWndCB, 0)
#define SHGetSubMenu(hWndCB,ID_MENU) (HMENU)GetSubMenu((HMENU)CommandBar_GetMenu(hWndCB, 0), ID_MENU)
#define SHSetSubMenu(hWndMB,ID_MENU)


#endif

#include <stdlib.h>

#include "files.h"


/*--------------------------------------------------------------------------+
| Globals
+--------------------------------------------------------------------------*/
SimplePlayer *Player;


/*----------------------------------------------------------------------
|       SimplePlayerWindowProc
+---------------------------------------------------------------------*/
LRESULT CALLBACK
SimplePlayerWindowProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
{
	SimplePlayer *player = Player;
    if (player) {
        return player->HandleWindowsMessage(window,
		                           message,
		 	    				   wparam,
								   lparam);
	} else {
		return DefWindowProc(window, message, wparam, lparam);
	}
}

/*----------------------------------------------------------------------
| HandleWindowsMessage
+---------------------------------------------------------------------*/
LRESULT
SimplePlayer::HandleWindowsMessage(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;

	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_OPEN:
					OpenFile();
					break;

				case IDM_EXIT:
					DestroyWindow(m_Window);
					break;
			}
			break;
		case WM_CREATE:
			break;
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				BeginPaint(hWnd, &ps);
				EndPaint(hWnd, &ps);
			}
			MtvpRepaint();
			break;
		case WM_SETFOCUS:
			MtvpFocus();
			break;
		case WM_DESTROY:
			MtvpQuit();
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

/*----------------------------------------------------------------------
| InitControls
+---------------------------------------------------------------------*/
int
SimplePlayer::InitControls()
{
	int status;
	HWND				menuCommandBar;		// The menu command bar handle

	INITCOMMONCONTROLSEX icex;
	icex.dwSize = sizeof(icex);
	icex.dwICC  = ICC_BAR_CLASSES | ICC_COOL_CLASSES;
	InitCommonControlsEx(&icex);

	SHMENUBARINFO mbi = {0};

	mbi.cbSize = sizeof (SHMENUBARINFO);
	mbi.hInstRes = m_Instance;
	mbi.hwndParent = m_Window;
	mbi.nToolBarId = IDM_MAIN;
	mbi.nBmpId = 0;
	mbi.cBmpImages = 0;
	mbi.dwFlags = SHCMBF_HIDESIPBUTTON;
	if (!SHCreateMenuBar(&mbi)) {
		menuCommandBar = NULL;
	} else {
		menuCommandBar = mbi.hwndMB;
	}

	if (!menuCommandBar) {
		status = GetLastError();
		_msg_printf("Cannot create menu bar (error %d)\n", status);
		return status ? status : -1;
	}

	return 0;
}


/*----------------------------------------------------------------------
| InitInstance
+---------------------------------------------------------------------*/
static SimplePlayer *
InitInstance(HINSTANCE hInstance, const TCHAR *window_class_name, const TCHAR *window_title, int nCmdShow, TCHAR *pCommandLine)
{
	HWND	hWnd;
	SimplePlayer *player;
	int status;

	{
		int cx, cy;

		int iDelta;
		SIPINFO si = {0};

		si.cbSize = sizeof(si);
		SHSipInfo(SPI_GETSIPINFO, 0, &si, 0);

		//Consider the menu at the bottom, please.
		iDelta = (si.fdwFlags & SIPF_ON) ? 0 : MENU_HEIGHT;
		cx = si.rcVisibleDesktop.right - si.rcVisibleDesktop.left;
		cy = si.rcVisibleDesktop.bottom - si.rcVisibleDesktop.top - iDelta;

		hWnd = CreateWindow(window_class_name, window_title, WS_VISIBLE,
				0, MENU_HEIGHT, cx, cy, NULL, NULL, hInstance, NULL);
		}
		if (hWnd == 0) {
			return NULL;
		}

	/*
	 * Start the player
	 */

	{
		static const char *mtvp_argv[] = {
			"mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
//			"-a0",	/* disable audio */
//			"-w",	/* trace everything */
//			"-aw12",
//			"-w3",
//			"-w11",
//			"-w12",
//			"-w4",
			NULL,	/* must be NULL-terminated */
		};

		player = new SimplePlayer(mtvp_argv, hInstance, hWnd);

		if (player == NULL) {
			ExitThread(0);
			return NULL;
		}
	}
	

	player->MtvpSetDisplayWindow((int)hWnd);

	ShowWindow(hWnd, nCmdShow);
	SetForegroundWindow(hWnd);

	status = player->InitControls();
	if (status != 0) {
		ExitThread(0);
		return NULL;
	}

	player->OpenFile();
//	player->MtvpOpenStreamURL("\\My Documents\\starwars.mpg");

	return player;
}


/*----------------------------------------------------------------------
| OpenFile
+---------------------------------------------------------------------*/
void
SimplePlayer::OpenFile()
{
	TCHAR name[MAX_PATH];
	USES_CONVERSION;

	if (ChooseOpenFile(m_Window, name, sizeofarray(name),
			FILE_CHOOSER_TYPE_MPEG_VIDEO, TEXT("Open MPEG File"))) {

		MtvpOpenStreamURL(W2A(name));
	}
}

/*----------------------------------------------------------------------
|       WinMain
+---------------------------------------------------------------------*/
int WINAPI
WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;
	const TCHAR	*window_title;			// The title bar text
	const TCHAR	*window_class_name;		// The window class name

	// Initialize global strings
	window_class_name = TEXT("SimplePlayer");
	window_title = TEXT("SimplePlayer");

	{
		WNDCLASS	wc = {0};

		wc.style			= CS_HREDRAW | CS_VREDRAW | CS_NOCLOSE;
		wc.lpfnWndProc		= (WNDPROC) SimplePlayerWindowProc;
		wc.hInstance		= hInstance;
		wc.hbrBackground	= (HBRUSH) GetStockObject(BLACK_BRUSH);
		wc.lpszClassName	= window_class_name;

		if (RegisterClass(&wc) == FALSE) return 0;
	}

	// Perform application initialization:
	Player = InitInstance (hInstance, window_class_name, window_title, nCmdShow, lpCmdLine);

	if (Player == NULL) {
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_SIMPLEPLAYER);


	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {

		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	
	return msg.wParam;
}

// eof
